<!doctype html>

<html>
    <head>
        <title>Redirecting to latest documentation</title>
        <meta charset="utf-8" />
        <meta
            http-equiv="refresh"
            content="0; url=./{{ latest.name }}/index.html"
        />
        <link rel="canonical" href="./{{ latest.name }}/index.html" />
    </head>
</html>
