"""
This module is part of the pysnmp library and is necessary to make the 'rfc7860.auth' directory a package.

Filepath:
    /Users/lextm/pysnmp.com/pysnmp/pysnmp/proto/secmod/rfc7860/auth/__init__.py
"""
