@import url("../pygments.css");
@import url("theme.css");

/* fix horizontal padding to accomodate adsense banners */
.wy-nav-content {
    padding: 1.618em 2.236em;
    height: 100%;
    /* max-width: 800px; */
    margin: auto;
}
